From c34c212815b40622b6cd29a314def2eaa7a5dd3e Mon Sep 17 00:00:00 2001
From: Ashutosh Bapat <ashutosh.bapat@enterprisedb.com>
Date: Tue, 10 Oct 2023 16:11:21 +0530
Subject: [PATCH 9/9] Handle infinite interval in
 brin_minmax_multi_distance_interval()

This also fixes code duplication between brin_minmax_multi_distance_interval() and interval_mi().

Adds testcase for testing infinite interval in BRIN indexes.
---
 src/backend/access/brin/brin_minmax_multi.c | 33 ++++-----------------
 src/test/regress/expected/brin_multi.out    |  9 +++---
 src/test/regress/sql/brin_multi.sql         | 10 ++++---
 3 files changed, 17 insertions(+), 35 deletions(-)

diff --git a/src/backend/access/brin/brin_minmax_multi.c b/src/backend/access/brin/brin_minmax_multi.c
index f8b2a3f9bc..61ebfb4542 100644
--- a/src/backend/access/brin/brin_minmax_multi.c
+++ b/src/backend/access/brin/brin_minmax_multi.c
@@ -2158,37 +2158,16 @@ Datum
 brin_minmax_multi_distance_interval(PG_FUNCTION_ARGS)
 {
 	float8		delta = 0;
-
-	Interval   *ia = PG_GETARG_INTERVAL_P(0);
-	Interval   *ib = PG_GETARG_INTERVAL_P(1);
 	Interval   *result;
-
 	int64		dayfraction;
 	int64		days;
 
-	result = (Interval *) palloc(sizeof(Interval));
-
-	result->month = ib->month - ia->month;
-	/* overflow check copied from int4mi */
-	if (!SAMESIGN(ib->month, ia->month) &&
-		!SAMESIGN(result->month, ib->month))
-		ereport(ERROR,
-				(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
-				 errmsg("interval out of range")));
-
-	result->day = ib->day - ia->day;
-	if (!SAMESIGN(ib->day, ia->day) &&
-		!SAMESIGN(result->day, ib->day))
-		ereport(ERROR,
-				(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
-				 errmsg("interval out of range")));
-
-	result->time = ib->time - ia->time;
-	if (!SAMESIGN(ib->time, ia->time) &&
-		!SAMESIGN(result->time, ib->time))
-		ereport(ERROR,
-				(errcode(ERRCODE_DATETIME_VALUE_OUT_OF_RANGE),
-				 errmsg("interval out of range")));
+	result = DatumGetIntervalP(DirectFunctionCall2(interval_mi,
+												   PG_GETARG_DATUM(1),
+												   PG_GETARG_DATUM(0)));
+
+	if (INTERVAL_NOT_FINITE(result))
+		PG_RETURN_FLOAT8(get_float8_infinity());
 
 	/*
 	 * Delta is (fractional) number of days between the intervals. Assume
diff --git a/src/test/regress/expected/brin_multi.out b/src/test/regress/expected/brin_multi.out
index 9f46934c9b..2b48069201 100644
--- a/src/test/regress/expected/brin_multi.out
+++ b/src/test/regress/expected/brin_multi.out
@@ -47,6 +47,7 @@ INSERT INTO brintest_multi (inetcol, cidrcol) SELECT
 	inet 'fe80::6e40:8ff:fea9:8c46' + tenthous,
 	cidr 'fe80::6e40:8ff:fea9:8c46' + tenthous
 FROM tenk1 ORDER BY thousand, tenthous LIMIT 25;
+INSERT INTO brintest_multi(intervalcol) VALUES ('-infinity'), ('+infinity');
 -- test minmax-multi specific index options
 -- number of values must be >= 16
 CREATE INDEX brinidx_multi ON brintest_multi USING brin (
@@ -217,9 +218,9 @@ INSERT INTO brinopers_multi VALUES
 	 '{1972-10-10 03:00:00-04, 1972-10-10 03:00:00-04, 1972-10-19 09:00:00-07, 1972-11-20 19:00:00-03, 1972-11-20 19:00:00-03}',
 	 '{100, 100, 1, 100, 100}'),
 	('intervalcol', 'interval',
-	 '{>, >=, =, <=, <}',
-	 '{00:00:00, 00:00:00, 1 mons 13 days 12:24, 2 mons 23 days 07:48:00, 1 year}',
-	 '{100, 100, 1, 100, 100}'),
+	 '{>, >=, =, =, <=, <}',
+	 '{00:00:00, 00:00:00, 1 mons 13 days 12:24, infinity, 2 mons 23 days 07:48:00, 1 year}',
+	 '{101, 101, 1, 1, 101, 101}'),
 	('timetzcol', 'timetz',
 	 '{>, >=, =, <=, <}',
 	 '{01:30:20+02, 01:30:20+02, 01:35:50+02, 23:55:05+02, 23:55:05+02}',
@@ -235,7 +236,7 @@ INSERT INTO brinopers_multi VALUES
 	('lsncol', 'pg_lsn',
 	 '{>, >=, =, <=, <, IS, IS NOT}',
 	 '{0/1200, 0/1200, 44/455222, 198/1999799, 198/1999799, NULL, NULL}',
-	 '{100, 100, 1, 100, 100, 25, 100}');
+	 '{100, 100, 1, 100, 100, 27, 100}');
 DO $x$
 DECLARE
 	r record;
diff --git a/src/test/regress/sql/brin_multi.sql b/src/test/regress/sql/brin_multi.sql
index d50dbdee68..b868b5bdb9 100644
--- a/src/test/regress/sql/brin_multi.sql
+++ b/src/test/regress/sql/brin_multi.sql
@@ -50,6 +50,8 @@ INSERT INTO brintest_multi (inetcol, cidrcol) SELECT
 	cidr 'fe80::6e40:8ff:fea9:8c46' + tenthous
 FROM tenk1 ORDER BY thousand, tenthous LIMIT 25;
 
+INSERT INTO brintest_multi(intervalcol) VALUES ('-infinity'), ('+infinity');
+
 -- test minmax-multi specific index options
 -- number of values must be >= 16
 CREATE INDEX brinidx_multi ON brintest_multi USING brin (
@@ -221,9 +223,9 @@ INSERT INTO brinopers_multi VALUES
 	 '{1972-10-10 03:00:00-04, 1972-10-10 03:00:00-04, 1972-10-19 09:00:00-07, 1972-11-20 19:00:00-03, 1972-11-20 19:00:00-03}',
 	 '{100, 100, 1, 100, 100}'),
 	('intervalcol', 'interval',
-	 '{>, >=, =, <=, <}',
-	 '{00:00:00, 00:00:00, 1 mons 13 days 12:24, 2 mons 23 days 07:48:00, 1 year}',
-	 '{100, 100, 1, 100, 100}'),
+	 '{>, >=, =, =, <=, <}',
+	 '{00:00:00, 00:00:00, 1 mons 13 days 12:24, infinity, 2 mons 23 days 07:48:00, 1 year}',
+	 '{101, 101, 1, 1, 101, 101}'),
 	('timetzcol', 'timetz',
 	 '{>, >=, =, <=, <}',
 	 '{01:30:20+02, 01:30:20+02, 01:35:50+02, 23:55:05+02, 23:55:05+02}',
@@ -239,7 +241,7 @@ INSERT INTO brinopers_multi VALUES
 	('lsncol', 'pg_lsn',
 	 '{>, >=, =, <=, <, IS, IS NOT}',
 	 '{0/1200, 0/1200, 44/455222, 198/1999799, 198/1999799, NULL, NULL}',
-	 '{100, 100, 1, 100, 100, 25, 100}');
+	 '{100, 100, 1, 100, 100, 27, 100}');
 
 DO $x$
 DECLARE
-- 
2.25.1

